package de.intevation.flys.exports;

import java.util.Date;
import java.util.List;

import org.apache.log4j.Logger;

import org.w3c.dom.Document;

import org.jfree.chart.JFreeChart;
import org.jfree.chart.axis.ValueAxis;
import org.jfree.chart.plot.XYPlot;
import org.jfree.data.Range;
import org.jfree.data.xy.DefaultXYDataset;

import de.intevation.artifacts.Artifact;

import de.intevation.artifactdatabase.state.Facet;

import de.intevation.flys.model.DischargeTable;
import de.intevation.flys.model.Gauge;
import de.intevation.flys.model.River;
import de.intevation.flys.model.TimeInterval;

import de.intevation.flys.artifacts.FLYSArtifact;

import de.intevation.flys.artifacts.model.WQKms;

/**
 * An OutGenerator that generates discharge curves.
 *
 * @author <a href="mailto:ingo.weinzierl@intevation.de">Ingo Weinzierl</a>
 */
public class DischargeCurveGenerator extends XYChartGenerator {

    /** The logger used in this generator.*/
    private static Logger logger =
        Logger.getLogger(DischargeCurveGenerator.class);


    public static final String I18N_CHART_TITLE =
        "chart.discharge.curve.title";

    public static final String I18N_CHART_SUBTITLE =
        "chart.discharge.curve.subtitle";

    public static final String I18N_XAXIS_LABEL =
        "chart.discharge.curve.xaxis.label";

    public static final String I18N_YAXIS_LABEL =
        "chart.discharge.curve.yaxis.label";

    public static final String I18N_CHART_TITLE_DEFAULT  = "Abflusskurven";
    public static final String I18N_XAXIS_LABEL_DEFAULT  = "Q [m\u00b3/s]";
    public static final String I18N_YAXIS_LABEL_DEFAULT  = "W [cm]";


    /** The Dataset that contains the data for the chart.*/
    protected DefaultXYDataset dataset;


    public DischargeCurveGenerator() {
        super();

        this.dataset = new DefaultXYDataset();
    }


    protected String getChartTitle() {
        return msg(I18N_CHART_TITLE, I18N_CHART_TITLE_DEFAULT);
    }


    @Override
    protected void addSubtitles(JFreeChart chart) {

    }


    protected String getXAxisLabel() {
        return msg(I18N_XAXIS_LABEL, I18N_XAXIS_LABEL_DEFAULT);
    }


    protected String getYAxisLabel() {
        return msg(I18N_YAXIS_LABEL, I18N_YAXIS_LABEL_DEFAULT);
    }


    @Override
    protected boolean zoomX(XYPlot plot, ValueAxis axis, Range range, Range x) {
        boolean zoomin = super.zoom(plot, axis, range, x);

        if (!zoomin) {
            axis.setLowerBound(0d);
        }

        return zoomin;
    }


    protected void addDatasets(JFreeChart chart) {
        XYPlot plot = (XYPlot) chart.getPlot();
        plot.setDataset(0, dataset);
    }


    public void doOut(Artifact artifact, Facet facet, Document attr) {
        logger.debug("DischargeCurveGenerator.doOut: " + facet.getName());

        if (!(artifact instanceof FLYSArtifact)) {
            logger.debug("Artifact is no instance of FLYSArtifact.");
            return;
        }

        FLYSArtifact flysArtifact = (FLYSArtifact) artifact;

        facet = flysArtifact.getNativeFacet(facet);

        if (facet == null) {
            logger.debug("no facet found");
            return;
        }

        WQKms wqkms = (WQKms)facet.getData(flysArtifact, context);

        String gaugeName = wqkms.getName();

        River river = flysArtifact.getRiver();

        if (river == null) {
            logger.debug("no river found");
            return;
        }

        Gauge gauge = river.determineGaugeByName(gaugeName);

        if (gauge == null) {
            logger.debug("no gauge found");
            return;
        }

        String seriesName = getSeriesName(gauge);

        double [][] values = new double [][] {
            wqkms.getQs(), wqkms.getWs() };

        dataset.addSeries(seriesName, values);
    }


    protected String getSeriesName(Gauge gauge) {
        // XXX The following code stops the artifact server accepting new HTTP
        // requests. It needs more analysis! Is loading the discharge table a
        // memory problem? Or is the time interval the problem?

        //List<DischargeTable> dts = gauge.getDischargeTables();

        //for (DischargeTable dt: dts) {
        //    if (dt.getKind() == 0) {
        //        TimeInterval ti = dt.getTimeInterval();

        //        Date start = ti.getStartTime();
        //        Date end   = ti.getStopTime();

        //        String name  = gauge.getName();

        //        if (end == null) {
        //            Object[] args = new Object[] { name, start };
        //            return msg(
        //                "chart.discharge.curve.curve.valid.from",
        //                "",
        //                args);
        //        }
        //        else {
        //            Object[] args = new Object[] { name, start, end };
        //            return msg(
        //                "chart.discharge.curve.curve.valid.range",
        //                "",
        //                args);
        //        }
        //    }
        //}

        return gauge.getName();
    }
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :
